package com.jenkins.demo.service;

import com.jenkins.demo.model.User;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.DisplayName;

import java.util.List;
import java.util.Optional;

import static org.junit.jupiter.api.Assertions.*;

/**
 * User Service Unit Tests
 */
class UserServiceTest {

    private UserService userService;

    @BeforeEach
    void setUp() {
        userService = new UserService();
    }

    @Test
    @DisplayName("应该能够获取所有用户")
    void shouldGetAllUsers() {
        List<User> users = userService.getAllUsers();
        assertNotNull(users);
        assertEquals(3, users.size()); // 初始化数据包含3个用户
    }

    @Test
    @DisplayName("应该能够根据ID获取用户")
    void shouldGetUserById() {
        Optional<User> user = userService.getUserById(1L);
        assertTrue(user.isPresent());
        assertEquals("admin", user.get().getUsername());
    }

    @Test
    @DisplayName("获取不存在的用户应该返回空")
    void shouldReturnEmptyForNonExistentUser() {
        Optional<User> user = userService.getUserById(999L);
        assertFalse(user.isPresent());
    }

    @Test
    @DisplayName("应该能够根据用户名查找用户")
    void shouldFindUserByUsername() {
        Optional<User> user = userService.getUserByUsername("admin");
        assertTrue(user.isPresent());
        assertEquals("admin", user.get().getUsername());
    }

    @Test
    @DisplayName("应该能够创建新用户")
    void shouldCreateNewUser() {
        User newUser = new User(null, "testuser", "test@example.com", "测试用户", null);
        User createdUser = userService.createUser(newUser);
        
        assertNotNull(createdUser.getId());
        assertEquals("testuser", createdUser.getUsername());
        assertEquals("test@example.com", createdUser.getEmail());
        assertEquals("测试用户", createdUser.getName());
        assertEquals("ACTIVE", createdUser.getStatus());
    }

    @Test
    @DisplayName("创建重复用户名应该抛出异常")
    void shouldThrowExceptionForDuplicateUsername() {
        User duplicateUser = new User(null, "admin", "admin2@example.com", "管理员2", null);
        
        IllegalArgumentException exception = assertThrows(
            IllegalArgumentException.class,
            () -> userService.createUser(duplicateUser)
        );
        
        assertTrue(exception.getMessage().contains("用户名已存在"));
    }

    @Test
    @DisplayName("应该能够更新用户信息")
    void shouldUpdateUser() {
        User updateData = new User(null, "admin_updated", "admin_new@example.com", "更新的管理员", null);
        Optional<User> updatedUser = userService.updateUser(1L, updateData);
        
        assertTrue(updatedUser.isPresent());
        assertEquals("admin_updated", updatedUser.get().getUsername());
        assertEquals("admin_new@example.com", updatedUser.get().getEmail());
        assertEquals("更新的管理员", updatedUser.get().getName());
    }

    @Test
    @DisplayName("更新不存在的用户应该返回空")
    void shouldReturnEmptyWhenUpdatingNonExistentUser() {
        User updateData = new User(null, "nonexistent", "test@example.com", "不存在的用户", null);
        Optional<User> result = userService.updateUser(999L, updateData);
        
        assertFalse(result.isPresent());
    }

    @Test
    @DisplayName("应该能够删除用户")
    void shouldDeleteUser() {
        boolean deleted = userService.deleteUser(1L);
        assertTrue(deleted);
        
        Optional<User> user = userService.getUserById(1L);
        assertFalse(user.isPresent());
    }

    @Test
    @DisplayName("删除不存在的用户应该返回false")
    void shouldReturnFalseWhenDeletingNonExistentUser() {
        boolean deleted = userService.deleteUser(999L);
        assertFalse(deleted);
    }

    @Test
    @DisplayName("应该能够获取用户总数")
    void shouldGetUserCount() {
        long count = userService.getUserCount();
        assertEquals(3, count);
    }

    @Test
    @DisplayName("应该能够激活用户")
    void shouldActivateUser() {
        boolean activated = userService.activateUser(1L);
        assertTrue(activated);
        
        Optional<User> user = userService.getUserById(1L);
        assertTrue(user.isPresent());
        assertEquals("ACTIVE", user.get().getStatus());
    }

    @Test
    @DisplayName("应该能够停用用户")
    void shouldDeactivateUser() {
        boolean deactivated = userService.deactivateUser(1L);
        assertTrue(deactivated);
        
        Optional<User> user = userService.getUserById(1L);
        assertTrue(user.isPresent());
        assertEquals("INACTIVE", user.get().getStatus());
    }
}
